class TextTag:
    """This class provides a set of SpaCy-style constants to use with Tag(kind="token")."""

    textWithWs = "textWithWs"
    whitespace = "whitespace"
    head = "head"
    leftEdge = "leftEdge"
    rightEdge = "rightEdge"
    entType = "entType"
    entIob = "entIob"
    lemma = "lemma"
    normalized = "normalized"
    shape = "shape"
    prefix = "prefix"
    suffix = "suffix"
    isAlpha = "isAlpha"
    isAscii = "isAscii"
    isDigit = "isDigit"
    isTitle = "isTitle"
    isPunct = "isPunct"
    isLeftPunct = "isLeftPunct"
    isRightPunct = "isRightPunct"
    isSpace = "isSpace"
    isBracket = "isBracket"
    isQuote = "isQuote"
    isCurrency = "isCurrency"
    likeUrl = "likeUrl"
    likeNum = "likeNum"
    likeEmail = "likeEmail"
    isOov = "isOov"
    isStop = "isStop"
    lang = "lang"
    embedding = "embedding"
